#include <windows.h>
#include <commctrl.h>

#pragma comment (linker, "/entry:Entry")
#pragma comment (linker, "/subsystem:windows")

#pragma comment (linker, "/merge:.rdata=.text")

#pragma comment (lib, "kernel32.lib")
#pragma comment (lib, "user32.lib")
#pragma comment (lib, "comctl32.lib")



class Instance {

  HINSTANCE const Inst;

  public:

  Instance (HINSTANCE const I) : Inst (I) { }

  operator HINSTANCE (void) const { return Inst; }

};



class Window {

  HWND Win;

  public:

  Window (void) { Win = 0; }

  ~Window (void) {

    if (Win) {

      DestroyWindow (Win);

    }

  }

  operator HWND (void) const { return Win; }

  static HWND GetDesktop (void) { return GetDesktopWindow (); }

  __forceinline bool Create (
    PCTSTR const ClassName, PCTSTR const WindowName, DWORD const Style,
    int const x, int const y, int const Width, int const Height,
    Instance const Inst
  ) {

    Win = CreateWindow (
      ClassName, WindowName, Style,
      x, y, Width, Height,
      GetDesktop (), 0, Inst, 0
    );

    return !!Win;

  }

  __forceinline void MessageLoop (void) const {

    MSG Msg;

    while (GetMessage (&Msg, Win, 0, 0) == TRUE) {

      TranslateMessage (&Msg);
      DispatchMessage (&Msg);

    }

  }

};





class EditWindow : public Window {

  public:

  EditWindow (void) { }

  __forceinline bool Create (Instance const Inst) {

    return Window::Create (
      "EDIT",
      0,
      (0
        | WS_OVERLAPPEDWINDOW
        | WS_VISIBLE
        | WS_VSCROLL
        | WS_HSCROLL
        | ES_MULTILINE
        //| ES_AUTOHSCROLL
        //| ES_AUTOVSCROLL
      ),
      500, 500, 300, 200,
      Inst
    );

  }

};






extern "C" DWORD WINAPI Entry (void const * const Arg) {

  (void) (Arg);

  Instance const Inst (GetModuleHandle (0));

  InitCommonControls ();

  EditWindow Edt;

  if (Edt.Create (Inst)) {

    Edt.MessageLoop ();

  }

  return 0;

}
